class FileAware:

    def files_list(self): ...

    def file_get(self, **kwargs): ...

    def upload(self, **kwargs): ...

    def download(self, **kwargs): ...

    def files_delete(self, **kwargs): ...

    def nuke(self): ...
